{% load i18n %}
<html>
<body>
<h1>django-getpaid dummy payment gateway</h1>


<p>
    {% blocktrans %}
    Are you authorizing payment "<strong>{{ payment }}</strong>"
    valued to <strong>{{ value }} {{ currency }}</strong>
    for "<strong>{{ description }}</strong>"?
    {% endblocktrans %}
</p>
<form method="post">
    {% csrf_token %}
    {{ form.as_p }}
    <input type="submit" value="{% trans "Continue" %}">
</form>

</body>
</html>
